@@ -1,10 +1,11 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1999-2001, 2005, 2007, 2009-2010 Free Software
+    Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,15 +14,13 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
 
 #include <parted/parted.h>
 #include <parted/debug.h>
-#include <parted/gnu.h>
 
 #include <errno.h>
 #include <hurd.h>
@@ -38,6 +37,23 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
+#include "../architecture.h"
+
+#define GNU_SPECIFIC(dev)	((GNUSpecific*) (dev)->arch_specific)
+
+typedef	struct _GNUSpecific	GNUSpecific;
+
+struct _GNUSpecific {
+	struct store*	store;
+	int consume;
+};
+
+/* Initialize a PedDevice using SOURCE.  The SOURCE will NOT be destroyed;
+   the caller created it, it is the caller's responsilbility to free it
+   after it calls ped_device_destory.  SOURCE is not registered in Parted's
+   list of devices.  */
+PedDevice* ped_device_new_from_store (struct store *source);
+
 static int
 _device_get_sector_size (PedDevice* dev)
 {
@@ -102,7 +118,7 @@ retry_open:
 			case PED_EXCEPTION_CANCEL:
 				goto error;
 		}
-	
+
 		return 0;
 	}
 
@@ -179,11 +195,11 @@ _init_device (const char *path)
 	return dev;
 
 error_free_arch_specific:
-	ped_free (dev->arch_specific);
+	free (dev->arch_specific);
 error_free_path:
-	ped_free (dev->path);
+	free (dev->path);
 error_free_dev:
-	ped_free (dev);
+	free (dev);
 error:
 	return NULL;
 }
@@ -199,9 +215,9 @@ _kernel_reread_part_table (PedDevice* dev)
 static void
 _done_device (PedDevice *dev)
 {
-        ped_free (dev->arch_specific);
-	ped_free (dev->path);
-	ped_free (dev);
+        free (dev->arch_specific);
+	free (dev->path);
+	free (dev);
 }
 
 /* Release all resources that libparted owns in DEV.  */
@@ -237,12 +253,12 @@ gnu_new (const char* path)
 	/* Try read-write. */
 	if (strchr (dev->path, '/') != NULL) {
 	        /* We set this to prevent having to use strchr more then once. */
-	        ispath = 1; 
+	        ispath = 1;
 
 	        rw_err = store_open (dev->path, 0, NULL, &arch_specific->store);
-	} else { 
+	} else {
 	        rw_err = store_typed_open (dev->path, 0, NULL, &arch_specific->store);
-	}        
+	}
 
 	/* Try readonly. */
 	if (rw_err) {
@@ -253,7 +269,7 @@ gnu_new (const char* path)
 	        ro_err = store_typed_open (dev->path, STORE_READONLY, NULL,
 				       &arch_specific->store);
 	  }
-                 
+
 	if (ro_err) {
 	        if (ped_exception_throw (
 			        PED_EXCEPTION_ERROR,
@@ -335,7 +351,7 @@ static int
 gnu_close (PedDevice* dev)
 {
 	GNUSpecific*	arch_specific = GNU_SPECIFIC (dev);
-	
+
 	_flush_cache (dev);
 
 	if (dev->dirty && dev->type != PED_DEVICE_FILE) {
@@ -573,7 +589,7 @@ doggy_first_block_read:
 doggy_first_block_write:
 		err = store_write (arch_specific->store,
 		    		   PED_TO_STORE (arch_specific->store, start),
-				   temp, arch_specific->store->block_size, 
+				   temp, arch_specific->store->block_size,
 				   &did_write);
 		if (! err && did_write != arch_specific->store->block_size)
 			err = EIO;
@@ -695,7 +711,7 @@ doggy_last_block_read:
 doggy_last_block_write:
 	err = store_write (arch_specific->store,
 	    		   PED_TO_STORE (arch_specific->store, start),
-			   temp, arch_specific->store->block_size, 
+			   temp, arch_specific->store->block_size,
 			   &did_write);
 	if (! err && did_write != arch_specific->store->block_size)
 		err = EIO;
@@ -754,7 +770,7 @@ gnu_sync (PedDevice* dev)
 
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (!dev->external_mode, return 0);
-	
+
 	arch_specific = GNU_SPECIFIC (dev);
 
 	if (dev->read_only || ! dev->dirty)
@@ -774,8 +790,7 @@ gnu_sync (PedDevice* dev)
 
 		switch (ex_status) {
 			case PED_EXCEPTION_IGNORE:
-				if (last_failure)
-					ped_free (last_failure);
+				free (last_failure);
 				last_failure = strdup (dev->path);
 				return 1;
 
@@ -787,7 +802,7 @@ gnu_sync (PedDevice* dev)
 			case PED_EXCEPTION_CANCEL:
 				return 0;
 		}
-	} 
+	}
 
 	return 1;
 }
@@ -872,4 +887,3 @@ PedArchitecture ped_gnu_arch = {
 	dev_ops:	&gnu_dev_ops,
 	disk_ops:	&gnu_disk_ops
 };
-
