@@ -1,6 +1,7 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2001-2002, 2005, 2007-2009 Free Software Foundation, Inc.
+    Copyright (C) 2001-2002, 2005, 2007-2010 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -94,19 +95,6 @@ dvh_probe (const PedDevice *dev)
 	return found;
 }
 
-#ifndef DISCOVER_ONLY
-static int
-dvh_clobber (PedDevice* dev)
-{
-	char *zeros = ped_calloc (dev->sector_size);
-	if (zeros == NULL)
-                return 0;
-	int ok = ped_device_write (dev, zeros, 0, 1);
-	free (zeros);
-	return ok;
-}
-#endif /* !DISCOVER_ONLY */
-
 static PedDisk*
 dvh_alloc (const PedDevice* dev)
 {
@@ -351,7 +339,6 @@ dvh_read (PedDisk* disk)
 	/* normal partitions */
 	for (i = 0; i < NPARTAB; i++) {
 		PedPartition* part;
-		PedConstraint* constraint_exact;
 
 		if (!vh.vh_pt[i].pt_nblks)
 			continue;
@@ -371,12 +358,14 @@ dvh_read (PedDisk* disk)
 		if (PED_BE16_TO_CPU (vh.vh_swappt) == i)
 			ped_partition_set_flag (part, PED_PARTITION_SWAP, 1);
 
-		constraint_exact = ped_constraint_exact (&part->geom);
-		if (!ped_disk_add_partition(disk, part, constraint_exact)) {
+		PedConstraint *constraint_exact
+		  = ped_constraint_exact (&part->geom);
+		bool ok = ped_disk_add_partition (disk, part, constraint_exact);
+		ped_constraint_destroy (constraint_exact);
+		if (!ok) {
 			ped_partition_destroy (part);
 			goto error_delete_all;
 		}
-		ped_constraint_destroy (constraint_exact);
 	}
 
 	if (!ped_disk_extended_partition (disk)) {
@@ -400,7 +389,6 @@ dvh_read (PedDisk* disk)
 	/* boot partitions */
 	for (i = 0; i < NVDIR; i++) {
 		PedPartition* part;
-		PedConstraint* constraint_exact;
 
 		if (!vh.vh_vd[i].vd_nbytes)
 			continue;
@@ -415,12 +403,14 @@ dvh_read (PedDisk* disk)
 		if (!strcmp (boot_name, ped_partition_get_name (part)))
 			ped_partition_set_flag (part, PED_PARTITION_BOOT, 1);
 
-		constraint_exact = ped_constraint_exact (&part->geom);
-		if (!ped_disk_add_partition(disk, part, constraint_exact)) {
+		PedConstraint *constraint_exact
+		  = ped_constraint_exact (&part->geom);
+		bool ok = ped_disk_add_partition (disk, part, constraint_exact);
+		ped_constraint_destroy (constraint_exact);
+		if (!ok) {
 			ped_partition_destroy (part);
 			goto error_delete_all;
 		}
-		ped_constraint_destroy (constraint_exact);
 	}
 #ifndef DISCOVER_ONLY
 	if (write_back)
@@ -862,7 +852,6 @@ dvh_alloc_metadata (PedDisk* disk)
 {
 	PedPartition* part;
 	PedPartition* extended_part;
-	PedConstraint* constraint_exact;
 	PedPartitionType metadata_type;
 	PED_ASSERT(disk != NULL, return 0);
 
@@ -879,67 +868,28 @@ dvh_alloc_metadata (PedDisk* disk)
 	if (!part)
 		goto error;
 
-	constraint_exact = ped_constraint_exact (&part->geom);
-	if (!ped_disk_add_partition (disk, part, constraint_exact))
-		goto error_destroy_part;
+	PedConstraint *constraint_exact
+	  = ped_constraint_exact (&part->geom);
+	bool ok = ped_disk_add_partition (disk, part, constraint_exact);
 	ped_constraint_destroy (constraint_exact);
-	return 1;
+	if (ok)
+		return 1;
 
-	ped_constraint_destroy (constraint_exact);
-error_destroy_part:
 	ped_partition_destroy (part);
 error:
 	return 0;
 }
 
-/*
- * Enforce some restrictions inherent in the dvh partition table format.
- * 1. Partition size must be smaller than 2^32 (unsigned int) sectors.
- *    If sector size is 512 bytes, this results in 2T aprox.
- * 2. Partition starting sector number must be smaller than 2^32.
- */
-static bool
-dvh_partition_check (const PedPartition* part)
-{
-	if (!ptt_partition_max_start_len("dvh", part))
-		return false;
-
-	return true;
-}
+#include "pt-common.h"
+PT_define_limit_functions (dvh)
 
 static PedDiskOps dvh_disk_ops = {
-	probe:			dvh_probe,
-#ifndef DISCOVER_ONLY
-	clobber:		dvh_clobber,
-#else
 	clobber:		NULL,
-#endif
-	alloc:			dvh_alloc,
-	duplicate:		dvh_duplicate,
-	free:			dvh_free,
-	read:			dvh_read,
-#ifndef DISCOVER_ONLY
-	write:			dvh_write,
-#else
-	write:			NULL,
-#endif
+	write:			NULL_IF_DISCOVER_ONLY (dvh_write),
 
-	partition_new:		dvh_partition_new,
-	partition_duplicate:	dvh_partition_duplicate,
-	partition_destroy:	dvh_partition_destroy,
-	partition_set_system:	dvh_partition_set_system,
-	partition_set_flag:	dvh_partition_set_flag,
-	partition_get_flag:	dvh_partition_get_flag,
-	partition_is_flag_available:	dvh_partition_is_flag_available,
 	partition_set_name:	dvh_partition_set_name,
 	partition_get_name:	dvh_partition_get_name,
-	partition_align:	dvh_partition_align,
-	partition_enumerate:	dvh_partition_enumerate,
-	partition_check:	dvh_partition_check,
-
-	alloc_metadata:		dvh_alloc_metadata,
-	get_max_primary_partition_count: dvh_get_max_primary_partition_count,
-	get_max_supported_partition_count: dvh_get_max_supported_partition_count
+	PT_op_function_initializers (dvh)
 };
 
 static PedDiskType dvh_disk_type = {
