@@ -1,7 +1,7 @@
 #!/bin/sh
 # Ensure that a simple command using -s succeeds with no prompt
 
-# Copyright (C) 2007, 2009 Free Software Foundation, Inc.
+# Copyright (C) 2007, 2009-2010 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ test_expect_success 'expect no output' 'compare out /dev/null'
 
 test_expect_success \
     'erase the left-over label' \
-    'dd if=/dev/zero of=$dev bs=4K count=1 2> /dev/null'
+    'dd if=/dev/zero of=$dev bs=4K count=1 conv=notrunc 2> /dev/null'
 
 # First iteration works with no prompting, since there is no preexisting label.
 test_expect_success \
